Graphics3D 800,600,32,2


; Player movement speed
Global PlayerSpeed#=0.5


; which layers to show initially?
Global sl1%=1
Global sl2%=1
Global sl3%=1
Global sl4%=1
Global sl5%=1


; Frametweening
Global GameSpeed%=60
Global Screenwidth%=GraphicsWidth()
Global Screenheight%=GraphicsHeight()
Global FramePeriod%=1000/GameSpeed
Global FrameTime%=MilliSecs()-FramePeriod


; move variables
Global Camera,SpeedMultiplicator%,DeltaTimeOld


; world
Global Pivot%,Licht%


; central pivot
Pivot=CreatePivot()


; Camera
Camera=CreateCamera()
PositionEntity Camera,-32,64,-64
CameraZoom Camera,1
CameraRange Camera,0.1,256
CameraFogMode Camera,True
CameraFogRange Camera,96,192
CameraFogColor Camera,155,174,206

CameraClsColor Camera,155,174,206
PointEntity Camera,Pivot


; Light
Licht=CreateLight(2)
PositionEntity Licht,10000,10000,10000
LightRange Licht,20000
AmbientLight 16,16,16


; Load maps
hm=LoadImage("maps/hm.png")						; Heightmap
lm=LoadImage("maps/lm.png")						; Lightmap
cm=LoadImage("maps/cm.png")						; Colormap

; Load Alpha maps
alpha_snow=LoadImage("maps/alpha_snow.png")		; Alphamap Snow
alpha_rock=LoadImage("maps/alpha_rock.png")		; Alphamap Rock
alpha_grass=LoadImage("maps/alpha_grass.png")	; Alphamap Grass
alpha_road=LoadImage("maps/alpha_road.png")		; Alphamap Road
alpha_smile=LoadImage("maps/alpha_smile.png")	; Alphamap Smile


; create Meshterrain
Terrain1=LoadMeshTerrain(hm,16,cm   ,1.0,alpha_grass) 				: EntityFX Terrain1,2
Terrain2=LoadMeshTerrain(hm,16,lm   ,1.0,alpha_rock)				: EntityFX Terrain2,2+32
Terrain3=LoadMeshTerrain(hm,16,False,0.0,alpha_snow ,255,255,255)	: EntityFX Terrain3,2+32
Terrain4=LoadMeshTerrain(hm,16,False,1.0,alpha_road ,  0,255,  0)	: EntityFX Terrain4,2+32
Terrain5=LoadMeshTerrain(hm,16,False,0.0,alpha_smile,192,192,128)	: EntityFX Terrain5,2+32


; Load Textures
tex1=MyLoadTexture("textures/grass.jpg"  ,0,2.0/ 32,1.0/ 32,5,Terrain1)
tex2=MyLoadTexture("textures/rock.jpg"   ,0,1.0/ 32,1.0/ 32,2,Terrain2)
tex3=MyLoadTexture("textures/snow.jpg"   ,0,1.0/ 16,1.0/ 16,2,Terrain3)
tex4=MyLoadTexture("textures/road.jpg"   ,0,1.0/128,1.0/128,5,Terrain4)
tex5=MyLoadTexture("textures/crystal.jpg",0,1.0/  4,1.0/  4,1,Terrain5)


; Apply detail texture
detail=MyLoadTexture("textures/detail.jpg" ,0,1.0/ 64,1.0/ 64,5)
EntityTexture terrain1,detail,0,2
EntityTexture terrain2,detail,0,2
EntityTexture terrain3,detail,0,2
EntityTexture terrain4,detail,0,2
EntityTexture terrain5,detail,0,2


; centered pointer
MoveMouse GraphicsWidth()/2,GraphicsHeight()/2


; check which layers to show/hide
If sl1=0 Then HideEntity terrain1 Else ShowEntity terrain1
If sl2=0 Then HideEntity terrain2 Else ShowEntity terrain2
If sl3=0 Then HideEntity terrain3 Else ShowEntity terrain3
If sl4=0 Then HideEntity terrain4 Else ShowEntity terrain4
If sl5=0 Then HideEntity terrain5 Else ShowEntity terrain5


; Main loop
While Not KeyHit(1)
	
	Cls
	
	; Frametween calculation
	Local FrameElapsed%,FrameTicks%,FrameTween#,t%
	Repeat FrameElapsed=MilliSecs()-FrameTime Until FrameElapsed
	FrameTicks=FrameElapsed/FramePeriod
	FrameTween=Float(FrameElapsed Mod FramePeriod)/Float(FramePeriod)
	
	; Frametween loop
	For t=1 To FrameTicks
		
		; Frametween Captureworld
		FrameTime=FrameTime+FramePeriod : If t=FrameTicks Then CaptureWorld
	
		; Space = Wireframe on/off
		If KeyHit(57) Then wf%=1-wf : WireFrame wf
		
		; Key 1-5 = show/hide Layer 1-5
		If KeyHit(2) Then sl1=1-sl1 : If sl1=0 Then HideEntity terrain1 Else ShowEntity terrain1
		If KeyHit(3) Then sl2=1-sl2 : If sl2=0 Then HideEntity terrain2 Else ShowEntity terrain2
		If KeyHit(4) Then sl3=1-sl3 : If sl3=0 Then HideEntity terrain3 Else ShowEntity terrain3
		If KeyHit(5) Then sl4=1-sl4 : If sl4=0 Then HideEntity terrain4 Else ShowEntity terrain4
		If KeyHit(6) Then sl5=1-sl5 : If sl5=0 Then HideEntity terrain5 Else ShowEntity terrain5
		
		; Movement
		FreeCam()
		
		UpdateWorld
		
	Next
	
	; Frametween Render
	RenderWorld FrameTween
	
	Text 0,0,TrisRendered()
	
	; Bufferflip
	Flip 0
	
Wend

End


; customized Loadtexture Function
Function MyLoadTexture(filename$,flags%,uscale#=1.0,vscale#=1.0,blendmode%=False,entity%=False,layer%=0,frame%=0)
	
	Local tex%=LoadTexture(filename$,flags)
	ScaleTexture tex,uscale,vscale
	If blendmode Then TextureBlend tex,blendmode
	If entity Then EntityTexture entity,tex,frame,layer
	
	Return tex
	
End Function


; create Meshterrain
Function LoadMeshTerrain(hmap%,vscale#=16.0,cmap%,cscale#=1.0,amap%,r1%=False,g1%=False,b1%=False)
	
	Local rgb%,r%,g%,b%,a#
	Local h#,x%,y%,vertex%
	Local vx#,vz#,u#,v#
	Local v0%,v1%,v2%,v3%
	
	; Heightmap
	If hmap Then
		Local hbuf%=ImageBuffer(hmap)
		Local size%=ImageWidth(hmap)-1
		Local verts%=size+1
	EndIf
	
	; Colormap
	If cmap Then 
		Local cbuf%=ImageBuffer(cmap)
		Local cwidth%=ImageWidth(cmap)
		Local cfactor#=cwidth*1.0/verts
	EndIf
	
	; Alphamap
	If amap Then Local abuf%=ImageBuffer(amap)
	
	; create the Mesh
	Local mesh%=CreateMesh()
	Local surf%=CreateSurface(mesh)
	
	; lock buffers
	LockBuffer hbuf
	If cmap Then LockBuffer cbuf
	If amap Then LockBuffer abuf
	
	For y=0 To size
		
		For x=0 To size
			
			; read height (only red channel) and normalize it
			h=Normalize((ReadPixelFast(x,size-y,hbuf) And $ff0000)/$10000,0,255,0.0,vscale)
			
			; read vertexcolor from colormap (R,G,B)
			If cmap Then
				rgb=ReadPixelFast(x*cfactor,(size-y)*cfactor,cbuf)
				r=Int(((rgb And $ff0000)/$10000)*cscale)
				g=Int(((rgb And $ff00)/$100)*cscale)
				b=Int((rgb And $ff)*cscale)
			Else
				r=128
				g=128
				b=128
			EndIf
			
			; read alpha value from alphamap (only red channel)
			If amap Then
				a=Normalize((ReadPixelFast(x,size-y,abuf) And $ff0000)/$10000,0,255,0,1)
			Else
				a=1.0
			EndIf
			
			; use forced colors if used
			If r1 Then r=r1
			If g1 Then g=g1
			If b1 Then b=b1
			
			; calculate vertex coordinates / texture coordinates
			vx=x-(size/2.0)
			vz=y-(size/2.0)
			u=x*1.0/size
			v=(size-y)*1.0/size
			
			; place vertex
			vertex=AddVertex(surf,vx,h,vz,u,v)
			
			; set vertex color and texture coordinates
			VertexColor surf,vertex,r,g,b,a
			
			; set triangles
			If y<size And x<size Then
				
				v0=x+((size+1)*y)
				v1=x+((size+1)*y)+(size+1)
				v2=(x+1)+((size+1)*y)
				v3=(x+1)+((size+1)*y)+(size+1)
				
				AddTriangle surf,v0,v1,v2
				AddTriangle surf,v2,v1,v3
				
			EndIf
			
		Next
		
	Next
	
	; unlock buffers
	If amap Then UnlockBuffer abuf
	If cmap Then UnlockBuffer cbuf
	UnlockBuffer hbuf
	
	UpdateNormals mesh
	
	Return mesh
	
End Function


; Freecam Movement
Function FreeCam(maxpitch#=85.0,rotspeed#=16.666,rotfloat#=8.0)
	
	Local movex#,movez#,dx#,dy#,dk#,dt%,t%
	Local movespeed#,pitch#
	
	SpeedMultiplicator=1
	
	; LMB/RMB = 2x faster
	If MouseDown(1) Or MouseDown(2) Then SpeedMultiplicator=2
	
	; Arrows = Move
	movex=KeyDown(205)-KeyDown(203)
	movez=KeyDown(200)-KeyDown(208)
	
	; Movespeed
	movespeed=PlayerSpeed*SpeedMultiplicator
	
	; smooth movement
	t=MilliSecs() : dt=t-DeltaTimeOld : DeltaTimeOld=t : dk=Float(dt)/rotspeed
	dx=(Screenwidth/2-MouseX())*0.01*dk : dy=(Screenheight/2-MouseY())*0.01*dk
	TurnEntity Camera,-dy,dx*0.1*dk*rotfloat,0
	
	; limit pitch
	pitch=EntityPitch(Camera,1) : If pitch>maxpitch Then pitch=maxpitch Else If pitch<-maxpitch Then pitch=-maxpitch
	
	; rotate and move
	RotateEntity Camera,pitch,EntityYaw(Camera,1),0,1	
	MoveEntity Camera,movex*movespeed,0,movez*movespeed
	
End Function


; Normalize a value
Function Normalize#(value#=128.0,value_min#=0.0,value_max#=255.0,norm_min#=0.0,norm_max#=1.0)
	
	Return ((value-value_min)/(value_max-value_min))*(norm_max-norm_min)+norm_min
	
End Function
;~IDEal Editor Parameters:
;~C#Blitz3D